=begin

    Discretizer, interactive mesh program. http://www.discretizer.org
    Copyright (C) 2009  Björn Bergqvist

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    
    If you want to contact the author send an email: bjorn.bergqvist@gmail.com or
    phone +46 704-80 11 83.

=end
# from cartesian coordinate system to spherical coordinate system
def cart2spherical(x,y,z)
   #begin
      x == 0 ? x = 1e-40:nil
      r = Math::sqrt(x**2 + y**2 + z**2)
      #fi = Math::atan(y/x)
      fi = solidAtan(x,y)
      # according to BETA theta = Math::acos(z/Math::sqrt(x**2+y**2+z**2))
      theta = Math::asin(z/Math::sqrt(x**2+y**2+z**2))
      # puts "fi #{fi}, theta #{theta}"
      return r, fi, toDeg(theta)
   #rescue
   #   raise RuntimeError, "Error while executing cart2spherical"
   #   return 0, 0, 0
   #end
end
# from spherical coordinate system to cartesian coordinate system
def spherical2cart(r, fi, theta)
   begin
      # It's not obvious which definition that is right here.
      x = r*Math::cos(toRad(theta))*Math::cos(toRad(fi))
      y = r*Math::cos(toRad(theta))*Math::sin(toRad(fi))
      z = r*Math::sin(toRad(theta))
      return x, y, z
   rescue
      raise RuntimeError, "Error while executing spherical2cart"
      return 0.0, 0.0, 0.0
   end
end
# from cartesian coordinate system to cylindrical coordinate system
def cart2cyl(x,y,z)
   begin
      x == 0 ? x = 1e-40:nil
      r = Math::sqrt(x**2 + y**2)
      #fi = Math::atan(y/x)
      fi = solidAtan(x,y)
      return r, fi, z
   rescue
      raise RuntimeError, "Error while executing cart2cyl"
      return 0.0, 0.0, 0.0
   end
end
# from cylindrical coordinate system to cartesian coordinate system
def cyl2cart(r,fi,z)
   begin
      x = r*Math::cos(toRad(fi))
      y = r*Math::sin(toRad(fi))
      return x, y, z
   rescue
      raise RuntimeError, "Error while executing cyl2cart"
      return 0.0, 0.0, 0.0
   end
end
def crossProduct(v1,v2)
   if v1.size == 3 && v2.size ==3
      return [v1[1].to_f*v2[2].to_f-v1[2].to_f*v2[1].to_f,-v1[0].to_f*v2[2].to_f+v1[2].to_f*v2[0].to_f,v1[0].to_f*v2[1].to_f-v1[1].to_f*v2[0].to_f]
   else
   puts "wrong size for cross product"
   end
end
def crossNormal(v1,v2)
   cross=crossProduct(v1,v2)
   length=vectLength(cross)
   return [cross[0]/length,cross[1]/length,cross[2]/length]
end
def faceNormal(tmpface)
   #begin # rescue
   if tmpface.class == DefFace && tmpface.info[0].class == InfoCube
      puts getpoint(tmpface, 0)
      puts getpoint(tmpface, 1)
      puts getpoint(tmpface, 2)
      puts getpoint(tmpface, 3)
      p0x, p0y, p0z = tmpface.info[0].p[getpoint(tmpface, 0)].pos[0]
      p1x, p1y, p1z = tmpface.info[0].p[getpoint(tmpface, 1)].pos[0]
      p3x, p3y, p3z = tmpface.info[0].p[getpoint(tmpface, 3)].pos[0]
   else
      p0x = tmpface.p[0].pos[0][0]
      p0y = tmpface.p[0].pos[0][1]
      p0z = tmpface.p[0].pos[0][2]
      p1x = tmpface.p[1].pos[0][0]
      p1y = tmpface.p[1].pos[0][1]
      p1z = tmpface.p[1].pos[0][2]
      p3x = tmpface.p[3].pos[0][0]
      p3y = tmpface.p[3].pos[0][1]
      p3z = tmpface.p[3].pos[0][2]
   end
   v0 = [ p1x - p0x, p1y - p0y, p1z - p0z ]
   v1 = [ p3x - p0x, p3y - p0y, p3z - p0z ]
   puts "p0x #{p0x}, p0y #{p0y}, p0z #{p0z}}"
   puts "p1x #{p1x}, p1y #{p1y}, p1z #{p1z}}"
   puts "p3x #{p3x}, p3y #{p3y}, p3z #{p3z}}"
   return crossNormal( v0, v1)
   ##begin # rescue
   ##rescue
   ##   puts "faceNormal method failed"
   ##   return [ 1.0, 0.0, 0.0 ]
   ##end #rescue end
end
def vectLength(v)
   return Math::sqrt(v[0].to_f**2+v[1]**2+v[2].to_f**2)
end
def crossSpecial(vec1,vec2,v)
  l=[vec2[0]-vec1[0],vec2[1]-vec1[1],vec2[2]-vec1[2]] 
  return crossNormal(l,v)
end
def angleVectorOld(vec1,ang)
   ang = ang*Math::PI/180.0
   # fan det blir bara 2-d nu...
   return [ Math::sin(Math::acos(vec1[1])+ang.to_f),Math::cos(Math::acos(vec1[1])+ang.to_f),0.0]
   # a2 = sin(asin(a)+v)
   # b2=cos(acos(b)+v)
end
def angleVector(vec1,ang)
   ang = ang*Math::PI/180.0
   # rotation kring z-axeln
   return [ vec1[0]*Math::cos(ang)-vec1[1]*Math::sin(ang),vec1[0]*Math::sin(ang)+vec1[1]*Math::cos(ang),0.0]
end
def scalarProd(vec1,vec2)
   return vec1[0].to_f*vec2[0].to_f+vec1[1].to_f*vec2[1].to_f+vec1[2].to_f*vec2[2].to_f
end
def scalarProduct(vec1,vec2)
   return vec1[0].to_f*vec2[0].to_f+vec1[1].to_f*vec2[1].to_f+vec1[2].to_f*vec2[2].to_f
end
def vertexDistance(v1,v2)
   return Math::sqrt( (v2[0]-v1[0])**2 +(v2[1]-v1[1])**2 +(v2[2]-v1[2])**2 ) 
end
def toDeg(rad)
   return rad*180.0/Math::PI
end
def toRad(deg)
   return deg*Math::PI/180.0
end
def rotateX(p,ang,v)
   # rotate v around a z-axis located at p
   mov = Vector.elements(p)
   rot = Vector.elements(v) - mov
   rot = Matrix[[1.0,0.0,0.0],
                [0.0,Math::cos(toRad(ang)), Math::sin(toRad(ang))],
                [0.0,-Math::sin(toRad(ang)),Math::cos(toRad(ang))]]*rot
   return rot + mov
end
def rotateXinv(p,ang,v)
   # rotate v around a z-axis located at p
   mov = Vector.elements(p)
   rot = Vector.elements(v) - mov
   rot = Matrix[[1.0,0.0,0.0],
                [0.0,Math::cos(toRad(ang)), -Math::sin(toRad(ang))],
                [0.0,Math::sin(toRad(ang)),Math::cos(toRad(ang))]]*rot
   return rot + mov
end
def rotateY(p,ang,v)
   # rotate v around a z-axis located at p
   mov = Vector.elements(p)
   rot = Vector.elements(v) - mov
   rot = Matrix[[Math::cos(toRad(ang)),0.0, -Math::sin(toRad(ang))],
                 [0.0,1.0,0.0],
                 [Math::sin(toRad(ang)),0.0,Math::cos(toRad(ang))]]*rot
   return rot + mov
end
def rotateYinv(p,ang,v)
   # rotate v around a z-axis located at p
   mov = Vector.elements(p)
   rot = Vector.elements(v) - mov
   rot = Matrix[[Math::cos(toRad(ang)),0.0, Math::sin(toRad(ang))],
                 [0.0,1.0,0.0],
                 [-Math::sin(toRad(ang)),0.0,Math::cos(toRad(ang))]]*rot
   return rot + mov
end
def rotateZ(p,ang,v)
   # rotate v around a z-axis located at p
   mov = Vector.elements(p)
   rot = Vector.elements(v) - mov
   rot = Matrix[[Math::cos(toRad(ang)), Math::sin(toRad(ang)),0.0],
                 [-Math::sin(toRad(ang)),Math::cos(toRad(ang)),0.0],
                 [0.0,0.0,1.0]]*rot
   return rot + mov
end
def rotateZinv(p,ang,v)
   # rotate v around a z-axis located at p
   mov = Vector.elements(p)
   rot = Vector.elements(v) - mov
   rot = Matrix[[Math::cos(toRad(ang)), -Math::sin(toRad(ang)),0.0],
                 [Math::sin(toRad(ang)),Math::cos(toRad(ang)),0.0],
                 [0.0,0.0,1.0]]*rot
   return rot + mov
end
def angleBetween(v1,v2)
   # return the angle between 2 vectors
   # cos(rad) = x scalarproduct y / |v1|*|v2|
   begin
      number = scalarProduct(v1,v2)/vectLength(v1)/vectLength(v2)
      number = 1.0 if number > 0.9999999999999
      number = -1.0 if number < -0.9999999999999
      acosnumber = Math::acos(number)
      todegnumber = toDeg(acosnumber)
      return todegnumber
      #return toDeg(Math::acos(scalarProduct(v1,v2)/vectLength(v1)/vectLength(v2)))
   rescue
      print "(fail: angle "
      print "v1=#{vectLength(v1)}, v2=#{vectLength(v2)}) "
      return 180.0
   end
end
def linearSpace(x1,y1,z1,x2,y2,z2,meshnum)
       if meshnum < 2
          meshnum = 2
       end
       # distance between points
       xrange = x2 -x1
       yrange = y2 -y1
       zrange = z2 -z1 
       # fill vectors with distances
       xpoints = Vector.elements( (0..(meshnum-1)).to_a )*(xrange.to_f/(meshnum-1).to_f)
       ypoints = Vector.elements( (0..(meshnum-1)).to_a )*(yrange.to_f/(meshnum-1).to_f)
       zpoints = Vector.elements( (0..(meshnum-1)).to_a )*(zrange.to_f/(meshnum-1).to_f)
       # adjust position of vectors so it starts at point @p0
       ones = Array.new
       (0..meshnum-1).each do |i|
          ones[i] = 1.0
       end
       x1pos = Vector.elements(ones)* x1.to_f
       y1pos = Vector.elements(ones)* y1.to_f
       z1pos = Vector.elements(ones)* z1.to_f

       xpoints += x1pos
       ypoints += y1pos
       zpoints += z1pos
       return xpoints, ypoints, zpoints
end
# test if normalized vectors are parallell
def isParallell(v1, v2)
   @v1 = normalizeVector(v1)
   @v2 = normalizeVector(v2)
   if @v1[0] == @v2[0] && @v1[1] == @v2[1] && @v1[2] == @v2[2]
      return true
   elsif (@v1[0]*0.9 < @v2[0] && @v1[0]*1.1 > @v2[0]) && (@v1[1]*0.9 < @v2[1] && @v1[1]*1.1 > @v2[1]) && (@v1[2]*0.9 < @v2[2] && @v1[2]*1.1 > @v2[2])
      return true
   else
      return false
   end
end
# normalize vector
def normalizeVector(v)
   @v = v
   @vlen = vectLength(@v)
   return [ @v[0]/@vlen, @v[1]/@vlen, @v[2]/@vlen ]
end
# find crossing of two lines starting at point pX with direction dX
def findCrossing(p1, d1, p2, d2)
   # first find out if they are parallell
   @p1, @d1, @p2, @d2 = p1, d1, p2, d2
   # solve in z-plane
   # from Wxmaxima: p1 + s1*d1 = p2 + s2*d2
   # s1=-(dx2*py2-dx2*py1+dy2*(px1-px2))/(dx1*dy2-dx2*dy1),
   # s2=-(dx1*py2-dx1*py1+dy1*(px1-px2))/(dx1*dy2-dx2*dy1)
   @s1= -(@d2[0]*@p2[1]-@d2[0]*@p1[1]+@d2[1]*(@p1[0]-@p2[0]))/(@d1[0]*@d2[1]-@d2[0]*@d1[1])
   @s2= -(@d1[0]*@p2[1]-@d1[0]*@p1[1]+@d1[1]*(@p1[0]-@p2[0]))/(@d1[0]*@d2[1]-@d2[0]*@d1[1])
   return [ @p1[0] + @s1*@d1[0], @p1[1] + @s1*@d1[1], @p1[2] + @s1*@d1[2] ]
end


# have xtable =  [0,1,2,3]
# and ytable = [8,7,5,9]
# get the linear interpolated value of y
def linearbetween(xtab,ytab,x)
   useidx=0 
   (1..xtab.size-1).each do |idx|
     useidx=idx if xtab[idx] < x
   end
   useidx=[useidx,xtab.size-2].min
   #puts "ytab #{ytab[useidx]} ytab+1 #{ytab[useidx+1]}"
   #puts "xtab #{xtab[useidx]} xtab+1 #{xtab[useidx+1]}"
   return ytab[useidx]+(ytab[useidx+1]-ytab[useidx]).to_f/(xtab[useidx+1]-xtab[useidx])*(x-xtab[useidx])
end


def transform2d(deg,rotx,roty,xinput,yinput)
   xtransformed=Array.new
   ytransformed=Array.new
   rad1=deg*Math::PI/180
   (0..xinput.size-1).each do |idx|
      xtmp=xinput[idx]-rotx
      ytmp=yinput[idx]-roty
      xtransformed[idx]=xtmp*Math.cos(rad1)-ytmp*Math.sin(rad1)+rotx
      ytransformed[idx]=xtmp*Math.sin(rad1)+ytmp*Math.cos(rad1)+roty
   end
   return xtransformed,ytransformed
end

def translate2d(movx,movy,xinput,yinput)
   xtranslated=Array.new
   ytranslated=Array.new
   (0..xinput.size-1).each do |idx|
      xtranslated[idx]=xinput[idx]+movx
      ytranslated[idx]=yinput[idx]+movy
   end
   return xtranslated,ytranslated
end

def scale2d(scalex,scaley,xinput,yinput)
   xscaled=Array.new
   yscaled=Array.new
   (0..xinput.size-1).each do |idx|
      xscaled[idx]=xinput[idx]*scalex
      yscaled[idx]=yinput[idx]*scaley
   end
   return xscaled,yscaled
end

def planetocyl(xtab,rtab,xinput,yinput)
   yret=Array.new
   zret=Array.new
   (0..xinput.size-1).each do |idx|
      radius=linearbetween(xtab,rtab,xinput[idx])
      fi=yinput[idx]/radius.to_f
      yret[idx]=radius*Math.sin(fi)
      zret[idx]=radius*Math.cos(fi)
   end 
   return xinput,yret,zret
end
