=begin

    Part of turbine model.
    Copyright (C) 2015  Björn Bergqvist

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    
    If you want to contact the author send an email: bjorn.bergqvist@gmail.com or
    phone +46 704-80 11 83.

=end
load 'naca.rb'
load 'cross.rb'
load 'stlcreate.rb'
begin
   $no_spliner=false
   require 'spliner'
rescue LoadError
   $no_spliner=true
end
#load 'equationnaca.rb'
#def naca65_008
#def naca65_010
#def naca63A008

def turbineblade(xrot,pitchmod)
x,y,dydx= naca65_010
x.pop
#x.pop
y.pop
#y.pop

tab_len=100
r1=80

rtab=[r1,200,300] #first value has to match rtab1:s first value
ttab=[0.9,0.9,0.7]

pitchtab=[-40.0-pitchmod,-50.0-pitchmod,-65.0-pitchmod]
cordatab=[200,200,200]
cambertab=[0.06,0.05,0.04]
rotx=[2,2,2]
roty=[0,0,0]
extrarottab=[-1,2.5,7.5]

xtab=[0,60]
rtab1=[r1,r1]
#rtab2=[103.2,118.8]
rtab2=[300,300]

xtop=Array.new
ytop=Array.new
ztop=Array.new
xbot=Array.new
ybot=Array.new
zbot=Array.new

if !$no_spliner then
   pitch_spline=Spliner::Spliner.new rtab,pitchtab
   thickness_spline=Spliner::Spliner.new rtab,ttab
   extrarot_spline=Spliner::Spliner.new rtab,extrarottab
end

sectioncount=30
x_end1=Array.new
y_end1=Array.new
z_end1=Array.new
x_end2=Array.new
y_end2=Array.new
z_end2=Array.new
(0..sectioncount-1).each do |sec|
   rtabtmp=Array.new
   (0..rtab1.size-1).each do |rtabidx|
      rtabtmp[rtabidx]=rtab1[rtabidx]+(rtab2[rtabidx]-rtab1[rtabidx])*sec/(sectioncount-1).to_f
   end
   radius=rtabtmp[0]
   if $no_spliner then
      thickness=linearbetween(rtab,ttab,radius)
      pitch=linearbetween(rtab,pitchtab,radius)
      extrarot=linearbetween(rtab,extrarottab,radius)
   else
      thickness=thickness_spline[radius.to_f]
      pitch=pitch_spline[radius.to_f]
      extrarot=extrarot_spline[radius]
   end

   cordalen=linearbetween(rtab,cordatab,radius)
   camberfrac=linearbetween(rtab,cambertab,radius)
   #camberfrac=camber.to_f/tab_len*cordalen/tab_len


   xtop1,ytop1,xbot1,ybot1 = camberstd(cordalen.to_f/tab_len,thickness,camberfrac,0.5,x,y,dydx)
   #puts "rtab #{rtab.join(" ")} pitchtab #{pitchtab.join(" ")} rtabtmp #{rtabtmp.join(" ")}"
   #pitch=eqpitch(radius)
   rotx1=linearbetween(rtab,rotx,radius)
   roty1=linearbetween(rtab,roty,radius)
   #puts pitch
   xtop2,ytop2=transform2d(pitch,rotx1,roty1,xtop1,ytop1)
   xbot2,ybot2=transform2d(pitch,rotx1,roty1,xbot1,ybot1)
   xtop[sec],ytop[sec],ztop[sec]=planetocyl(xtab,rtabtmp,xtop2,ytop2)
   xbot[sec],ybot[sec],zbot[sec]=planetocyl(xtab,rtabtmp,xbot2,ybot2)

   #extra x rotation
   ytop[sec],ztop[sec]=transform2d(extrarot+xrot,0,0,ytop[sec],ztop[sec])
   ybot[sec],zbot[sec]=transform2d(extrarot+xrot,0,0,ybot[sec],zbot[sec])

   #collect endpoints
   x_end1[sec]=xbot[sec].last
   y_end1[sec]=ybot[sec].last
   z_end1[sec]=zbot[sec].last
   x_end2[sec]=xtop[sec].last
   y_end2[sec]=ytop[sec].last
   z_end2[sec]=ztop[sec].last

end
stl=""
(0..sectioncount-2).each do |sec|
   stl+=stlbetween(xtop[sec],ytop[sec],ztop[sec],xtop[sec+1],ytop[sec+1],ztop[sec+1])
   stl+=stlbetween(xbot[sec],ybot[sec],zbot[sec],xbot[sec+1],ybot[sec+1],zbot[sec+1])
end
closeend=stlbetween(x_end1,y_end1,z_end1,x_end2,y_end2,z_end2)
closebot=stlbetween(xtop[0],ytop[0],ztop[0],xbot[0],ybot[0],zbot[0])
closetop=stlbetween(xtop[sectioncount-1],ytop[sectioncount-1],ztop[sectioncount-1],xbot[sectioncount-1],ybot[sectioncount-1],zbot[sectioncount-1])
   stl+=closebot
   stl+=closetop
   stl+=closeend
   return stl
end
=begin
puts stlheader("turbineblade")
puts turbineblade(0) 
puts turbineblade(30) 
puts stlfooter("turbineblade")
=end
